@@ -17,7 +17,7 @@ require Exporter;
 
 @ISA       = qw(Exporter);
 @EXPORT_OK = qw(encrypt decrypt encrypt_hex decrypt_hex);
-$VERSION   = '2.15';
+$VERSION   = '2.20';
 
 
 ########################################
@@ -33,7 +33,7 @@ sub new ($;$$$)
 
 	my $self =
 	{
-		padding		=> 'standard', # default method
+		padding		=> 'standard', # default padding method
 		mode		=> '',
 		key		=> '',
 		cipher		=> '',
@@ -71,6 +71,8 @@ sub new ($;$$$)
 			$options->{cipher} = shift || 'DES';
 		}
 
+		# cipher has to be called before keysize and blocksize
+		# otherwise it would override values provided by the user
 		$self->$_( $options->{$_} ) foreach qw(cipher keysize key blocksize padding);
 	}
 
@@ -80,10 +82,10 @@ sub new ($;$$$)
 #
 # set attributes if argument given, return attribute value
 #
-sub module	(\$)	{                                      return $_[0]->{module}    }
-sub keysize	(\$;$)	{ $_[0]->{keysize}   = $_[1] if $_[1]; return $_[0]->{keysize}   }
-sub blocksize	(\$;$)	{ $_[0]->{blocksize} = $_[1] if $_[1]; return $_[0]->{blocksize} }
-sub mode	(\$)	{                                      return $_[0]->{mode}      }
+sub module	(\$)	{					return $_[0]->{module}    }
+sub keysize	(\$;$)	{ $_[0]->{keysize}   = $_[1] if $_[1];	return $_[0]->{keysize}   }
+sub blocksize	(\$;$)	{ $_[0]->{blocksize} = $_[1] if $_[1];	return $_[0]->{blocksize} }
+sub mode	(\$)	{ 					return $_[0]->{mode}      }
 
 #
 # sets key if argument given
@@ -116,20 +118,22 @@ sub padding (\$;$)
 		# if given a custom padding...
 		if (ref $padding eq 'CODE')
 		{
-			# make sure a block cipher has been loaded, otherwise we can't check the padding code
-			die "Can't check custom padding without a cipher. Use '\$ecb->cipher(\$cipher)',"
-			  . "  \$cipher being some algorithm like for example 'DES', 'IDEA' or 'Blowfish'."
-			  . " Of course, the corresponding module 'Crypt::\$cipher' needs to be installed.\n"
-				unless $self->{module};
-
-			my $bs = $self->{blocksize};
-
-			# ...check whether it works as expected
-			for my $i (0 .. $bs-1)
+			# ...for different block sizes...
+			for my $bs ((8, 16))
 			{
-				my $len = length($padding->(' ' x $i, $bs, 'e') || '');
-				die "Provided padding method does not behave properly: Expected $bs bytes, got $len bytes.\n"
-					unless ($len == $bs);
+				# ...check whether it works as expected
+				for my $i (0 .. $bs-1)
+				{
+					my $plain = ' ' x $i;
+
+					my $padded = $padding->($plain, $bs, 'e') || '';
+					die "Provided padding method does not pad properly: Expected $bs bytes, got ", length $padded, ".\n"
+						unless (length $padded == $bs);
+
+					my $trunc = $padding->($padded, $bs, 'd') || '';
+					die "Provided padding method does not truncate properly: Expected '$plain', got '$trunc'.\n"
+						unless ($trunc eq $plain);
+				}
 			}
 		}
 
@@ -254,7 +258,7 @@ sub crypt (\$;$)
 	my $self = shift;
 	my $data = shift;
     
-	$data = ($_ || '') unless (defined $data && length $data);
+	$data = ($_ || '') unless defined $data;
 
 	my $bs     = $self->{blocksize};
 	my $mode   = $self->{mode};
@@ -289,7 +293,7 @@ sub finish (\$)
 	my $data   = $self->{_buffer};
 	my $result = '';
 
-	die "You tried to use crypt() without calling start() before. Use '\$ecb->start(\$mode)'"
+	die "You tried to use finish() without calling start() before. Use '\$ecb->start(\$mode)'"
 	  . " first, \$mode being one of 'decrypt' or 'encrypt'.\n"
 		unless $mode;
 
@@ -350,17 +354,19 @@ sub _pad (\$$)
 	my $bs      = $self->{blocksize};
 	my $padding = $self->{padding};
 
+	my $pad = $bs - length $data;
+
 	my $message = "Your message length is not a multiple of $self->{cipher}'s blocksize ($bs bytes)."
 		    . " Correct this by hand or tell me to handle padding.\n";
 
-	$padding eq 'standard' ?	$data .= chr($bs-length $data) x ($bs-length $data)		:
-	$padding eq 'zeroes' ?		$data .= "\0"x($bs-length($data)-1) . chr($bs-length $data)	:
-	$padding eq 'oneandzeroes' ?	                   $data .= "\x80"."\0"x($bs-length($data)-1)	:
-	$padding eq 'rijndael_compat' ?	(length $data) && ($data .= "\x80"."\0"x($bs-length($data)-1))	:
-	$padding eq 'null' ?		                   $data .=        "\0"x($bs-length $data)	:
-	$padding eq 'space' ?		(length $data) && ($data .=        " " x($bs-length $data))	:
-	ref $padding eq 'CODE' ?	$data = $padding ->($data, $bs, 'e')				:
-	$padding eq 'none' ?		(length($data) % $bs) && die $message				:
+	$padding eq 'standard' ?	$data .= chr($pad) x  $pad				:
+	$padding eq 'zeroes' ?		$data .=      "\0" x ($pad-1) . chr($pad)		:
+	$padding eq 'oneandzeroes' ?	                   $data .= "\x80" . "\0"x($pad-1)	:
+	$padding eq 'rijndael_compat' ?	(length $data) && ($data .= "\x80" . "\0"x($pad-1))	:
+	$padding eq 'null' ?		                   $data .=          "\0"x $pad		:
+	$padding eq 'space' ?		(length $data) && ($data .=          " " x $pad)	:
+	ref $padding eq 'CODE' ?	$data = $padding ->($data, $bs, 'e')			:
+	$padding eq 'none' ?		(length($data) % $bs) && die $message			:
 
 	# still here?
 	die "Padding style '$padding' not defined.\n";
@@ -379,18 +385,33 @@ sub _truncate (\$$)
 	my $bs      = $self->{blocksize};
 	my $padding = $self->{padding};
 
-	$padding eq 'standard' ?	substr($data, -ord(substr $data, -1)) = ''	:
-	$padding eq 'zeroes' ?		substr($data, -ord(substr $data, -1)) = ''	:
-	$padding eq 'random' ?		substr($data, -ord(substr $data, -1)) = ''	:
-	$padding eq 'oneandzeroes' ?	$data =~ s/\x80\0*$//s				:
-	$padding eq 'rijndael_compat' ?	$data =~ s/\x80\0*$//s				:
-	$padding eq 'null' ?		$data =~ s/\0+$//s				:
-	$padding eq 'space' ?		$data =~ s/ +$//s				:
-	ref $padding eq 'CODE' ?	$data = $padding->($data, $bs, 'd')		:
-	$padding eq 'none' ?		()						:
+	if ($padding =~ /^(standard|zeroes|random)$/)
+	{
+		my $trunc = ord(substr $data, -1);
 
-	# still here?
-	die "Padding style '$padding' not defined.\n";
+		die "Asked to truncate $trunc bytes, which is greater than $self->{cipher}'s blocksize ($bs bytes).\n"
+			if $trunc > $bs;
+
+		my $expected =	$padding eq 'standard' ?	chr($trunc) x  $trunc			:
+				$padding eq 'zeroes' ?		       "\0" x ($trunc-1) . chr($trunc)	:
+				$padding eq 'random' ?	substr($data, -$trunc, $trunc-1) . chr($trunc)	: 'WTF!?';
+
+		die "Block doesn't look $padding padded.\n" unless $expected eq substr($data, -$trunc);
+
+		substr($data, -$trunc) = '';
+	}
+	else
+	{
+		$padding eq 'oneandzeroes' ?	$data =~ s/\x80\0*$//s			:
+		$padding eq 'rijndael_compat' ?	$data =~ s/\x80\0*$//s			:
+		$padding eq 'null' ?		$data =~ s/\0+$//s			:
+		$padding eq 'space' ?		$data =~ s/ +$//s			:
+		ref $padding eq 'CODE' ?	$data = $padding->($data, $bs, 'd')	:
+		$padding eq 'none' ?		()					:
+
+		# still here?
+		die "Padding style '$padding' not defined.\n";
+	}
 
 	return $data;
 }
@@ -415,7 +436,7 @@ sub _crypt
 	{
 		($mode, $key, $cipher, $data, $padding) = @_;
 
-		$self = Crypt::ECB->new($key => $cipher);
+		$self = __PACKAGE__->new($key => $cipher);
 		$self->padding($padding) if $padding;
 
 		$data = $_ unless length($data);
@@ -713,11 +734,12 @@ the encrypted information into an e-mail message, web page or URL.
 
 For convenience en- or decrypting can also be done by calling ordinary
 functions. The functions are: B<encrypt()>, B<decrypt()>,
-B<encrypt_hex()>, B<decrypt_hex()>. The module is smart enough to
-recognize whether these functions are called in an OO context or not.
+B<encrypt_hex()>, B<decrypt_hex()>.
 
 =head2 encrypt(), decrypt(), encrypt_hex(), decrypt_hex()
 
+  use Crypt::ECB qw(encrypt decrypt encrypt_hex decrypt_hex);
+
   $ciphertext = encrypt($key, $cipher, $plaintext, $padstyle);
   $plaintext  = decrypt($key, $cipher, $ciphertext, $padstyle);
 
@@ -730,8 +752,9 @@ and padstyle are optional. If the padding style is omitted, 'standard'
 is assumed. If data is omitted, $_ is used.
 
 B<encrypt_hex()> and B<decrypt_hex()> operate on ciphertext in a
-hexadecimal representation. Otherwise usage is the same as for
-B<encrypt()> and B<decrypt()>.
+hexadecimal representation, just like the methods with the same name,
+see above. Otherwise usage is the same as for B<encrypt()> and
+B<decrypt()>.
 
 =head1 BUGS
 
@@ -741,15 +764,19 @@ None that I know of. Please report if you find any.
 
 Implement 'random' padding, see http://www.di-mgt.com.au/cryptopad.html.
 
-Convenience encrypt and decrypt functions utilizing base64 encoding
-could be added.
-
 A taint check on the key like Crypt::CBC does could be added.
 
 =head1 LICENSE
 
-This module is distributed under the "Artistic License" using the same
-terms as Perl itself. 
+Crypt-ECB is Copyright (C) 2000, 2005, 2008, 2016 by Christoph Appel.
+
+This module is distributed using the same terms as Perl itself. It is free
+software; you can redistribute it and/or modify it under the terms of either:
+
+a) the GNU General Public License as published by the Free Software
+Foundation; either version 1, or (at your option) any later version, or
+
+b) the "Artistic License".
 
 =head1 AUTHOR
 
